h3 {
    border-bottom: 1px solid #ddd;
}

.top-bar {
    height: 45px;
    min-height: 45px;
    position: absolute;
    top: 0;
    right: 0;
    left: 0;
}

.bars-lnk {
    color: #fff;
}

.bars-lnk i {
    display: inline-block;
    margin-left: 10px;
    margin-top: 7px;
}

.bars-lnk img {
    display: inline-block;
    margin-left: 10px;
    margin-top: -15px;
    margin-right: 15px;
    height: 35px;
}

.lateral-menu {
    background-color: #333;
    color: rgb(144, 144, 144);
    width: 300px;
    
}

.lateral-menu label {
    color: rgb(144, 144, 144);
}

.lateral-menu-content {
    padding-left: 10px;
    height: 100%;
    font-size: 12px;
    font-style: normal;
    font-variant: normal;
    font-weight: bold;
    line-height: 16px;
}

.lateral-menu-content .title{
    padding-top: 15px;
    font-size: 2em;
    height: 45px;
}

.lateral-menu-content-inner {
    overflow-y: auto;
    height: 100%;
    padding-top: 10px;
    padding-bottom: 50px;
    padding-right: 10px;
    font-size: 0.9em;
}








#preview {
    height: 97%;
    max-height: 97%;
    border: 1px solid #eee;
    overflow-y: scroll;
    width: 55%;
    padding: 10px;
}

pre {
    white-space: pre-wrap;       /* css-3 */
    white-space: -moz-pre-wrap;  /* Mozilla, since 1999 */
    white-space: -pre-wrap;      /* Opera 4-6 */
    white-space: -o-pre-wrap;    /* Opera 7 */
    word-wrap: break-word;       /* Internet Explorer 5.5+ */
    background-color: #f8f8f8;
    border: 1px solid #dfdfdf;
    margin-top: 1.5em;
    margin-bottom: 1.5em;
    padding: 0.125rem 0.3125rem 0.0625rem;
}

pre code {
    background-color: transparent;
    border: 0;
    padding: 0;
}


.modal-wrapper {
    position: absolute;
    width: 100%;
    height: 100%;
    top: 0;
    left: 0;
    z-index: 999;
    background-color: rgba(51,51,51,0.5);
}

.modal-inner {
    margin-top: 200px;
    margin-left: auto;
    margin-right: auto;
    width: 600px;
    height: 225px;
    background-color: #fff;
    opacity: 1;
    z-index: 1000;
}

.modal-close-btn {
    float: right;
    display: inline-block;
    margin-right: 5px;
    color: #ff4336;
}

.modal-close-btn:hover {
    float: right;
    display: inline-block;
    margin-right: 5px;
    color: #8d0002;
}

.modal-topbar {
    clear: both;
    height: 25px;
}

.modal-inner .link-area {
    margin: 10px;
    height: 170px;

}

.modal-inner textarea {
    width: 100%;
    height: 100%;
    margin: 0;
    padding: 0;
}

.version {
    color: white;
    font-size: 0.8em !important;
}