{% from "macro_menu.html" import menu, menu_module, menu_page, make_setting_menu_page with context %}
{% from "macro_include.html" import apply_theme, modals with context %}
{% import "macro.html" as macros %}
<!DOCTYPE html>
<html>
  <head lang="ko">
    {% block head %}
    <title>{{get_web_title()}}</title>
    {% endblock %}
    <meta name="google" value="notranslate">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">

    <link rel="shortcut icon" href="{{ url_for('static', filename='img/favicon.ico') }}">
    <link href="{{ url_for('static', filename='css/bootstrap.min.css') }}" rel="stylesheet">
    {% set theme = get_theme() %}
    <link href="{{ url_for('static', filename='css/theme/'+theme+'_bootstrap.min.css') }}" rel="stylesheet">                
    <link href="{{ url_for('static', filename='css/animate.min.css') }}" rel="stylesheet">
    <link href="{{ url_for('static', filename='css/custom.css') }}" rel="stylesheet">
    <link href="https://unpkg.com/balloon-css/balloon.min.css" rel="stylesheet">
    <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js"></script>
    <script src="https://code.jquery.com/ui/1.12.1/jquery-ui.js"></script>
    <!--<script src="{{ url_for('static', filename='js/popper.min.js') }}"></script>-->

    <script src="{{ url_for('static', filename='js/bootstrap.min.js') }}"></script>
    <script src="{{ url_for('static', filename='js/bootstrap-notify.min.js') }}"></script>    

    <script src="{{ url_for('static', filename='js/ff_common1.js') }}"></script>
    <script src="{{ url_for('static', filename='js/ff_ui1.js') }}"></script>


    <!--<script type="text/javascript" src="https://cdnjs.cloudflare.com/ajax/libs/socket.io/1.4.6/socket.io.js"></script>-->
    <script type="text/javascript" src="https://cdnjs.cloudflare.com/ajax/libs/socket.io/4.5.0/socket.io.js"></script>
    <!-- 토글 -->
    <link href="https://cdn.jsdelivr.net/gh/gitbrent/bootstrap4-toggle@3.4.0/css/bootstrap4-toggle.min.css" rel="stylesheet">
    <script src="https://cdn.jsdelivr.net/gh/gitbrent/bootstrap4-toggle@3.4.0/js/bootstrap4-toggle.min.js"></script>
    <!-- end 토글 -->

    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.min.css">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/jquery-contextmenu/2.8.0/jquery.contextMenu.min.css">
    <script src="https://cdnjs.cloudflare.com/ajax/libs/jquery-contextmenu/2.8.0/jquery.contextMenu.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/jquery-contextmenu/2.8.0/jquery.ui.position.js"></script>


  </head>

  <body class="body ">
    {{ menu() }}
    {{ menu_module() }}

    <!--</nav>-->
    <main id="main_container" role="main" class="container">
      <div class="d-inline-block"></div>
        {{ menu_page() }}
        {{ make_setting_menu_page() }}
      <div>
        {% block content %}{% endblock %}
      </div>
    </main>
    <div class="loading" id="loading">
      <img src="/static/img/loading.gif" />
    </div>
    {{ modals() }}
  </body>
</html>

<!-- 글로벌 버튼이 모두 나오고 처리-->
<script src="{{ url_for('static', filename='js/sjva_global1.js') }}"></script>
<script src="{{ url_for('static', filename='js/ff_global1.js') }}"></script>
<script src="{{ url_for('static', filename='js/ff_global_plugin.js') }}"></script>
